//
//  Function.h
//  FinotesCore
//
//  Created by Don Peter on 12/1/17.
//  Copyright © 2017 BasilGregory Software Labs Pvt Ltd. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import "Observer.h"
#import <CoreLocation/CoreLocation.h>

@interface Fn : NSObject

//Function to initialize Finotes.
+(void) initialize:(UIApplication *) application;

+(void) initializeMe;

//Internal API.
+(void) preventNetworkMonitoring;
//Internal API.
+(void) preventActivityTrailTracking;
//Internal API.
+(void) preventErrorMonitoring;

//Function to report custom issues that can be anticipated. e.g payment failure, error in video playback
+(void) reportIssueAt:(id) target withShortDescription:(NSString *) shortDescription withDescription:(NSString *) description withSeverity:(int64_t) severity;

+(void) reportIssueAt:(NSString *) target withShortDescription:(NSString *) shortDescription withDescription:(NSString *) description withStack:(NSString *) stack;

//Function to report custom issues that can be anticipated. e.g payment failure, error in video playback
+(void) reportIssueAt:(id) target withShortDescription:(NSString *) shortDescription withDescription:(NSString *) description ;

//Leaks are now auto tracked.
+(void) watchLeaks:(UIViewController *) viewController;

//Error message is optional.
+ (void) onNetworkCallCompletedWithRequest:(NSURLRequest *) request
                               andResponse:(NSHTTPURLResponse *) response
                           andResponseData:(NSData *) responseData
                              withOptionalErrorMessage: (NSString *) errorMessage;


#pragma Utility APIs


//Function to set activity markers, All activity markers set for a minute pior to an issue detection will be sent along with the dashboard.
//Can set any number of activity markers in project code.
+(void) setActivityMarkerAt:(id) target marker:(NSString *) marker ;

//Sets debug level activity marker.
//Will be active only in FinotesDebug flavour of Framework
//Function to set activity markers, All activity markers set for a minute pior to an issue detection will be sent along with the dashboard.
//Can set any number of activity markers in project code.
+(void) setActivityMarkerForDebugAt:(id) target marker:(NSString *) marker ;

+(void) setActivityMarkerAtString:(NSString *) target marker:(NSString *) marker;

//Issue raise frequency in Hours.
+(void) setIssueRaiseFrequencyInHours:(int8_t) numberOfHours __deprecated;

//Tag your users with any custom identifier (e.g SAD12, SAD12 (Premium User), SAD12 (Trial User))
+(void) updateUserDataWithUniqueIdentifier:(NSString *) uniqueIdentifier __deprecated;
+(void) setCustomId:(NSString *) uniqueIdentifier;

//Once implemented in AppDelegate, anytime an issue is raised from finotes SEL function will be invoked.
+(void) listenForIssues:(SEL) selector atTarget:(id) target ;
//Funtion to set location, The lastest location data set using this API will be tagged with issue reports
+(void) updateLocation:(CLLocation *) location ;

//To prevent stacktrace from being sent up to the server.
+(void) excludeStack;
//Function to prevent issue reports being sent to dashboard, ideally to be used in development mode.
+(void) dryRun;
+(void) log;
+(void) test;

#pragma Function trackers


+(Observer*) observe ;


+(id) call:(SEL) methodSelector target:(id) targetOfMethod observer:(Observer *) observer withParameters:(id) parameter, ...;
+(id) call:(SEL) methodSelector target:(id) targetOfMethod withParameters:(id) parameter, ...;
+(id) call:(SEL) methodSelector target:(id) targetOfMethod observer:(Observer *) observer ;
+(id) call:(SEL) methodSelector target:(id) targetOfMethod;

+(id) call:(SEL) methodSelector target:(id) targetOfMethod observer:(Observer *) observer withParams:(NSArray *) parameters;
+(id) call:(SEL) methodSelector target:(id) targetOfMethod withParams:(NSArray *) parameters;



+(void) start:(Observer *) observer withParams:(NSArray *) params;
+(void) start:(Observer *) observer;
+(void) start;

+(void) end:(id) returnValue;
+(void) end;


+(void) setAPIResponse:(NSDictionary *) response;
+(void) setAPIRequest:(NSDictionary *) request;

+(void) buildExceptionWithLineNumber:(NSString *) lineNumber At:(NSString *) location withStackTrace:(NSString *) stackTrace andExceptionObject:(NSString *) exception;

+(void) buildCrashWithLineNumber:(NSString *) lineNumber At:(NSString *) location withStackTrace:(NSString *) stackTrace andExceptionObject:(NSString *) exception;

+(void) buildMemoryLeakIssueIn:(NSString *) location withDetails:(NSString *) locationWithDetails;

+(void) buildAppHangIssue:(NSString *) currentScreen withThreshold:(double) threshold withBackStack:(NSString *) stacktrace;

@end
